<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:t="http://software.in2p3.fr/lavoisier/tables.xsd"
                exclude-result-prefixes="xalan">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>

    <xsl:variable name="debug" select="false()" />

    <xsl:variable name="parameters" select="document('options')/options"/>

    <xsl:variable name="options" select="$parameters/sections"/>


    <xsl:template match="/t:tables">
        <xsl:if test="$debug">
            <xsl:message terminate="no">Debug from ChartMapping.xsl</xsl:message>
            <xsl:message terminate="no">    options <xsl:copy-of select="$options" /></xsl:message>
            <xsl:message terminate="no">    Input <xsl:copy-of select="/" /></xsl:message>
        </xsl:if>
        <supermap>
            <meta>
                <property name="title" type="string"><xsl:value-of select="@title" /></property>
            </meta>
            <xsl:for-each select="t:table">
                <map>
                    <!-- TODO: Penser à faire les vérification avant de créer les attributs au cas ou un template aurait un retour vide. -->
                    <xsl:apply-templates select="$options"> <xsl:with-param name="table" select="." /> </xsl:apply-templates>
                </map>
            </xsl:for-each>
        </supermap>
    </xsl:template>

    <xsl:template match="sections">
        <xsl:param name="table" />
        <xsl:if test="section[@id='series']">
            <xsl:apply-templates select="section[@id='series']"> <xsl:with-param name="table" select="$table" /> </xsl:apply-templates>
        </xsl:if>
        <xsl:if test="section[@id='yAxis']">
            <property name="yAxis" type="array">
                <xsl:apply-templates select="section[@id='yAxis']">
                    <xsl:with-param name="table" select="$table" />
                </xsl:apply-templates>
            </property>
        </xsl:if>
        <xsl:if test="section[@id='plotOptions']">
            <property name="plotOptions" type="object"> <xsl:apply-templates select="section[@id='plotOptions']" /> </property>
        </xsl:if>
        <!--<xsl:if test="not(section[@id='subtitle'] and /t:tables/t:fields)">
            <xsl:apply-templates select="." mode="SUBTITLE" />
        </xsl:if>-->
        <xsl:apply-templates select="section[@id='xAxis']"> <xsl:with-param name="table" select="$table" /> </xsl:apply-templates>
        <xsl:if test="not(section[@id='title'])"><xsl:apply-templates select="$table/@name" /></xsl:if>
        <xsl:apply-templates select="./*[not(@id='series' or @id='xAxis' or @id='yAxis' or @id='plotOptions')]" />
    </xsl:template>
    <xsl:template match="section[@id='xAxis']">
        <xsl:param name="table" />
        <property name="xAxis" type="array">
            <object>
                <xsl:choose>
                    <xsl:when test="./option[@name='categories' and @mode='column']">
                        <property name="categories" type="array">
                            <xsl:apply-templates select="./option[@name='categories']"> <xsl:with-param name="table" select="$table" /> </xsl:apply-templates>
                        </property>
                    </xsl:when>
                    <xsl:when test="./option[@name='categories' and @mode='unit']">
                        <xsl:apply-templates select="./option[@name='categories']"> <xsl:with-param name="table" select="$table" /> </xsl:apply-templates>
                    </xsl:when>
                    <xsl:when test="./option[@name='categories' and @mode='list']"> <xsl:apply-templates select="./option[@name='categories']" /> </xsl:when>
                    <xsl:otherwise>
                        <xsl:apply-templates select="./option[@name='categories']">
                            <xsl:with-param name="table" select="$table" />
                        </xsl:apply-templates>
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:apply-templates select="./*[not(@name='categories')]" />
            </object>
        </property>
    </xsl:template>
    <xsl:template match="section[@id='yAxis' and not(@mode)]">
        <object> <xsl:apply-templates /> </object>
    </xsl:template>
    <!-- Using this template we will create the yAxis' from all different unit there are -->
    <xsl:template match="section[@id='yAxis' and @mode='unit']">
        <xsl:param name="table" />

        <xsl:variable name="units" select="$table/t:column_labels/t:column_label[@unit and not(preceding-sibling::t:column_label/@unit = @unit)]" />
        <xsl:for-each select="$units">
            <xsl:variable name="unit" select="@unit" />
            <object>
                <xsl:if test="not(opposite) and position() div 2 = floor(position() div 2)">
                    <property name="opposite" type="boolean">true</property>
                </xsl:if>
                <property name="name" type="string"><xsl:value-of select="$unit" /></property>
            </object>
        </xsl:for-each>
    </xsl:template>
    <!-- Using this template we will take the data from the column where the column_label is what were given by the configuration -->
    <xsl:template match="section[@id='series' and @name and @mode='column']">
        <xsl:param name="table" />
        <xsl:variable name="name" select="@name" />

        <object>
            <xsl:apply-templates select="./*[not(@id='data' or @name='name')]" />
            <property name="data" type="array">
                <xsl:for-each select="$table/t:row">
                    <number>
                        <xsl:value-of select="./t:column[count($table/t:column_labels/t:column_label[text()=$name]/preceding-sibling::t:column_label)+1]/text()" />
                    </number>
                </xsl:for-each>
            </property>
            <property name="name" type="string">
                <xsl:choose>
                    <xsl:when test="./option[@name='name']"> <xsl:value-of select="./option[@name='name']/text()" /> </xsl:when>
                    <xsl:when test="@name"> <xsl:value-of select="@name" /> </xsl:when>
                </xsl:choose>
            </property>
        </object>
    </xsl:template>
    <!-- Using this template we will take the data from the column where the column_label do not have a unit attribute -->
    <xsl:template match="section[@id='series' and @name and @mode='unit']|section[@id='series' and @name and not(@mode)]">
        <xsl:param name="table" />
        <xsl:variable name="name" select="@name" />

        <object>
            <xsl:apply-templates select="./*[not(@id='data' or @name='name')]" />
            <property name="name" type="string"><xsl:value-of select="$name" /></property>
            <property name="data" type="array">
                <xsl:for-each select="$table/t:row">
                    <number>
                        <xsl:choose>
                            <xsl:when test="$table/t:column_labels/t:column_label[@unit=$name]">
                                <xsl:value-of select="./t:column[count($table/t:column_labels/t:column_label[@unit=$name]/preceding-sibling::t:column_label)+1]/text()" />
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="./t:column[count($table/t:column_labels/t:column_label[@unit]/preceding-sibling::t:column_label)+1]/text()" />
                            </xsl:otherwise>
                        </xsl:choose>
                    </number>
                </xsl:for-each>
            </property>
        </object>
    </xsl:template>
    <!-- Using this template we will create all series from column_label with unit attribute -->
    <xsl:template match="section[@id='series' and not(@name) and @mode='unit']">
        <xsl:param name="table" />

        <xsl:variable name="units" select="$table/t:column_labels/t:column_label[@unit and not(preceding-sibling::t:column_label/@unit = @unit)]" />
        <property name="series" type="array">
            <xsl:for-each select="$table/t:column_labels/t:column_label[@unit]">
                <xsl:variable name="unit" select="@unit" />
                <object>
                    <property name="name" type="string"><xsl:value-of select="." /></property>
                    <property name="data" type="array">
                        <xsl:for-each select="$table/t:row/t:column[count(current()/preceding-sibling::t:column_label)+1]">
                            <number><xsl:value-of select="text()" /></number>
                        </xsl:for-each>
                    </property>
                    <property name="yAxis" type="number">
                        <xsl:apply-templates select="@unit" mode="POSITION">
                            <xsl:with-param name="units" select="$units" />
                        </xsl:apply-templates>
                    </property>
                </object>
            </xsl:for-each>
        </property>
    </xsl:template>
    <xsl:template match="section[@id='plotOptions']" priority="1">
        <property name="{@type}" type="object"> <xsl:apply-templates select="./*" /> </property>
    </xsl:template>
    <!-- Add a title to the table with its name -->
    <xsl:template match="t:table/@name">
        <property name="title" type="object">
            <property name="text" type="string"><xsl:value-of select="." /></property>
        </property>
    </xsl:template>

    <!-- Using this template we will take all category from the table content where the column is at the same position that the column_label contained in the entries (node 'entry') -->
    <xsl:template match="option[@name='categories' and @mode='column']">
        <xsl:param name="table" />
        <xsl:variable name="option" select="." />
        <xsl:for-each select="$table/t:row">
            <xsl:variable name="row" select="." />
            <string>
                <xsl:for-each select="$option/entry">
                    <xsl:variable name="name" select="text()" />
                    <xsl:if test="position()>1"> <!-- separator between column --><xsl:value-of select="'_'" /> </xsl:if>
                    <xsl:value-of select="$row/t:column[count($table/t:column_labels/t:column_label[text()=$name]/preceding-sibling::t:column_label)+1]/text()" />
                </xsl:for-each>
            </string>
        </xsl:for-each>
    </xsl:template>
    <!-- Using this template we will 'paste' the value given as the category value -->
    <xsl:template match="option[@name='categories' and @mode='list']">
        <property name="categories"> <xsl:value-of select="text()" /> </property>
    </xsl:template>
    <!-- Using this template we will take all category from the table content where the column_label do not have a unit attribute -->
    <xsl:template match="option[@name='categories' and @mode='unit']|option[@name='categories' and not(@mode)]">
        <xsl:param name="table" />
        <property name="categories" type="array">
            <xsl:for-each select="$table/t:row">
                <xsl:variable name="row" select="." />
                <string>
                    <xsl:for-each select="ancestor-or-self::t:table/t:column_labels/t:column_label[not(@unit)]">
                        <xsl:if test="position()>1">
                            <!-- separator between column --><xsl:value-of select="'_'" />
                        </xsl:if>
                        <xsl:value-of select="$row/t:column[count(current()/preceding-sibling::t:column_label)+1]/text()" />
                    </xsl:for-each>
                </string>
            </xsl:for-each>
        </property>
    </xsl:template>

    <!-- Taking subtitle from /tables/fields/field -->
    <xsl:template match="*" mode="SUBTITLE">
        <property name="subtitle" type="object">
            <property name="text" type="string">
                <xsl:for-each select="/t:tables/t:fields/t:field">
                    <xsl:if test="position()>1"><xsl:value-of select="', '" /></xsl:if>
                    <xsl:value-of select="text()" />
                </xsl:for-each>
            </property>
        </property>
    </xsl:template>
    <!-- Taking the unit position from all different units in the table -->
    <xsl:template match="@unit" mode="POSITION">
        <xsl:param name="units" />

        <xsl:variable name="unit" select="." />
        <xsl:for-each select="$units">
            <xsl:if test="@unit=$unit">
                <xsl:value-of select="position()-1" />
            </xsl:if>
        </xsl:for-each>
    </xsl:template>

    <!-- Additional 'section' and 'option' -->
    <xsl:template match="section" priority="-1">
        <xsl:choose>
            <xsl:when test="./section[@id='item']">
                <property name="{@id}" type="array">
                    <xsl:for-each select="./section[@id='item']">
                        <object> <xsl:apply-templates select="./*" /> </object>
                    </xsl:for-each>
                </property>
            </xsl:when>
            <xsl:when test="./option[@name='item']">
                <property name="{@id}" type="array"><xsl:apply-templates select="*" /></property>
            </xsl:when>
            <xsl:otherwise> <property name="{@id}" type="object"> <xsl:apply-templates select="*" /> </property> </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="option[not(@name='item')]" priority="-1">
        <property name="{@name}">
            <xsl:choose>
                <xsl:when test="number(text())=text()"><xsl:attribute name="type"><xsl:value-of select="'number'" /></xsl:attribute></xsl:when>
                <xsl:when test="text()='true' or text()='false'"> <xsl:attribute name="type"><xsl:value-of select="'boolean'" /></xsl:attribute></xsl:when>
                <xsl:when test="not(text()='null' or text()='undefined')"> <xsl:attribute name="type"><xsl:value-of select="'string'" /></xsl:attribute></xsl:when>
            </xsl:choose>
            <xsl:value-of select="text()" />
        </property>
    </xsl:template>
    <xsl:template match="option[@name='item']" priority="-1">
        <xsl:choose>
            <xsl:when test="number(text())=text()"> <number> <xsl:value-of select="text()" /> </number> </xsl:when>
            <xsl:when test="text()='true' or text()='false'"> <boolean><xsl:value-of select="text()" /> </boolean> </xsl:when>
            <xsl:when test="not(text()='null' or text()='undefined')"> <string> <xsl:value-of select="text()" /> </string> </xsl:when>
            <xsl:otherwise> <value> <xsl:value-of select="text()" /> </value> </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>
